function w = myhamming(fr,Br,fs,alpha)
% MYHAMMING  Weighting function for SAR spectra (range).
%    MYHAMMING(FR) computes hamming window for frequencies in 
%    vector FR (MHz, e.g. from -FS/2 to +FS/2).
%
%    MYHAMMING(FR,BR,FS,A) uses specified parameters.
%    Default for sampling frequency (range sampling rate)
%    FS=18.96 MHz, range band width BR=15.55 MHz, and alpha
%    A=0.75 (ERS1, DPAF).
%
%    This window is defined by (Geudtner, Curlander):
%       W(Fr) = [a + (1-a)*cos(2pi*fr/fs)] * rect(fr/Br).
%       Where: rect(x)=1 for abs(x)<.5
%    Note: rect not periodic, thus myhamming neither.
%    (which is ok for range filtering, not for azimuth.)
%
%    E.g. to compute hamming window at frequencies fr,
%       N  = 128;  fs = 18.96e6;  Bw = 15.55e6;  df = fs/(N);
%       fr = -fs/2:df:fs/2-df;
%       h  = myhamming(fr,Bw,fs,0.75);
%       plot(fr,h,'b'); xlabel('freq [Hz]')
%
%   A more bandlimited shifted (centered) window can be generated by:
%       sh = 10;%				shift in pixels
%       h2 = myhamming(fr-.5*sh*df,Bw-abs(sh)*df,fs,0.75);
%       hold on; plot(fr,h2,'r'); hold off;
%       title('myhamming windows for SAR spectral filtering.');
%    MYHAMMING is used to test for the Doris software.
%    It is used in adaptive range filtering.
%
%    See also MYRECT, BOXCAR, HAMMING, RANGEDEMO, WSHIFT.
%


% $Revision: 1.4 $  $Date: 2000/12/07 16:29:12 $
% Bert Kampes, 16/03/00

if (nargin<4) alpha  =  0.75; end;
if (nargin<3) fs     = 18.96; end;
if (nargin<2) Br     = 15.55; end;
if (nargin<1) error('Please specify interval.'); end;
if (size(fr,1)~=1 & size(fr,2)~=1) error('fr vector only.'); end;

%
w = (alpha + (1.-alpha).*cos((2.*pi/fs).*fr)) .* myrect(fr./Br);


%---------------------------------------------------
%function r=myrect(x)
% r=zeros(size(x));
% r(find(abs(x)<=.5))=1;
%%% EOF
